function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { ConcurrentError } from 'resolve-storage-base';
const DUPLICATE_KEY_ERROR = 11000;

const saveEvent = async (pool, event) => {
  const {
    database,
    collectionName,
    isFrozen
  } = pool;

  if (await isFrozen()) {
    throw new Error('Event store is frozen');
  }

  try {
    const collection = await database.collection(collectionName);
    const currentThreadId = Math.floor(Math.random() * 256);
    const nextThreadCounter = ~~Object((await collection.find({
      threadId: currentThreadId
    }).sort({
      threadCounter: -1
    }).project({
      threadCounter: 1
    }).toArray())[0]).threadCounter + 1;
    await collection.insertOne(_objectSpread({}, event, {
      timestamp: Math.max(event.timestamp, Date.now()),
      threadId: currentThreadId,
      threadCounter: nextThreadCounter
    }));
  } catch (error) {
    const errorCode = error != null && error.code != null ? error.code : 0;
    const errorMessage = error != null && error.message != null ? error.message : '';

    if (errorCode === DUPLICATE_KEY_ERROR && errorMessage.indexOf('aggregate') > -1) {
      throw new ConcurrentError(event.aggregateId);
    } else if (errorCode === DUPLICATE_KEY_ERROR && errorMessage.indexOf('thread') > -1) {
      return await saveEvent(pool, event);
    } else {
      throw error;
    }
  }
};

export default saveEvent;
//# sourceMappingURL=save-event.js.map