"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const sortExpression = {
  timestamp: 1,
  threadCounter: 1
};
const projectionExpression = {
  _id: 0,
  threadId: 0,
  threadCounter: 0
};

const loadEventsByTimestamp = async ({
  database,
  collectionName,
  shapeEvent
}, {
  eventTypes,
  aggregateIds,
  startTime,
  finishTime,
  limit
}, callback) => {
  const batchSize = limit != null ? limit : 0x7fffffff;

  const findExpression = _objectSpread({}, eventTypes != null ? {
    type: {
      $in: eventTypes
    }
  } : {}, {}, aggregateIds != null ? {
    aggregateId: {
      $in: aggregateIds
    }
  } : {}, {
    timestamp: {
      $gt: startTime != null ? startTime : 0,
      $lt: finishTime != null ? finishTime : Infinity
    }
  });

  const collection = await database.collection(collectionName);
  const cursorStream = collection.find(findExpression).sort(sortExpression).project(projectionExpression).stream();
  let lastError = null;

  for (let event = await cursorStream.next(), eventsLeft = batchSize; event != null && eventsLeft-- > 0; event = await cursorStream.next()) {
    try {
      await callback(shapeEvent(event));
    } catch (error) {
      lastError = error;
      break;
    }
  }

  await cursorStream.close();

  if (lastError != null) {
    throw lastError;
  }
};

var _default = loadEventsByTimestamp;
exports.default = _default;
//# sourceMappingURL=load-events-by-timestamp.js.map